import { Inject, Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import {
  LoggerBase,
  MessagePublisherPort,
  PrismaRepositoryBase,
} from '@mobicoop/ddd-library';
import { PrismaService } from './prisma.service';
import { UserEntity } from '../core/domain/user.entity';
import { UserRepositoryPort } from '../core/application/ports/user.repository.port';
import { UserMapper } from '../user.mapper';
import { USER_MESSAGE_PUBLISHER } from '../user.di-tokens';
import { SERVICE_NAME } from '@src/app.constants';

export type UserBaseModel = {
  uuid: string;
  firstName?: string;
  lastName?: string;
  email?: string;
  phone?: string;
};

export type UserReadModel = UserBaseModel & {
  createdAt: Date;
  updatedAt: Date;
};

export type UserWriteModel = UserBaseModel;

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class UserRepository
  extends PrismaRepositoryBase<UserEntity, UserReadModel, UserWriteModel>
  implements UserRepositoryPort
{
  constructor(
    prisma: PrismaService,
    mapper: UserMapper,
    eventEmitter: EventEmitter2,
    @Inject(USER_MESSAGE_PUBLISHER)
    protected readonly messagePublisher: MessagePublisherPort,
  ) {
    super(
      prisma.user,
      prisma,
      mapper,
      eventEmitter,
      new LoggerBase({
        logger: new Logger(UserRepository.name),
        domain: SERVICE_NAME,
        messagePublisher,
      }),
    );
  }
}
