import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { NotFoundException } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { RpcValidationPipe } from '@mobicoop/ddd-library';
import { UserMapper } from '@modules/user/user.mapper';
import { FindUserByIdRequestDto } from './dtos/find-user-by-id.request.dto';
import { UserResponseDto } from '../dtos/user.response.dto';
import { UserEntity } from '@modules/user/core/domain/user.entity';
import { FindUserByIdQuery } from '@modules/user/core/application/queries/find-user-by-id/find-user-by-id.query';
import { GRPC_SERVICE_NAME } from '@src/app.constants';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class FindUserByIdGrpcController {
  constructor(
    protected readonly mapper: UserMapper,
    private readonly queryBus: QueryBus,
  ) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'FindOneById')
  async findOnebyId(data: FindUserByIdRequestDto): Promise<UserResponseDto> {
    try {
      const user: UserEntity = await this.queryBus.execute(
        new FindUserByIdQuery(data.id),
      );
      return this.mapper.toResponse(user);
    } catch (e) {
      if (e instanceof NotFoundException) {
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: e.message,
        });
      }
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
