import { Test, TestingModule } from '@nestjs/testing';
import { UserEntity } from '@modules/user/core/domain/user.entity';
import { FindUserByIdQueryHandler } from '@modules/user/core/application/queries/find-user-by-id/find-user-by-id.query-handler';
import { USER_REPOSITORY } from '@modules/user/user.di-tokens';
import { FindUserByIdQuery } from '@modules/user/core/application/queries/find-user-by-id/find-user-by-id.query';

const now = new Date('2023-06-21 06:00:00');
const user: UserEntity = new UserEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df44',
  props: {
    firstName: 'John',
    lastName: 'Doe',
    email: 'john.doe@email.com',
    phone: '+33611223344',
  },
  createdAt: now,
  updatedAt: now,
});

const mockUserRepository = {
  findOneById: jest.fn().mockImplementation(() => user),
};

describe('find-user-by-id.query-handler', () => {
  let findUserByIdQueryHandler: FindUserByIdQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USER_REPOSITORY,
          useValue: mockUserRepository,
        },
        FindUserByIdQueryHandler,
      ],
    }).compile();

    findUserByIdQueryHandler = module.get<FindUserByIdQueryHandler>(
      FindUserByIdQueryHandler,
    );
  });

  it('should be defined', () => {
    expect(findUserByIdQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return a user', async () => {
      const findUserbyIdQuery = new FindUserByIdQuery(
        'dd264806-13b4-4226-9b18-87adf0ad5dd1',
      );
      const user: UserEntity =
        await findUserByIdQueryHandler.execute(findUserbyIdQuery);
      expect(user.getProps().lastName).toBe('Doe');
    });
  });
});
